class Api

  constructor: (name, version) ->
    # build path
    @name = name or 'api'
    @version = parseInt(version) or 1
    @path = "/#{@name}/v#{@version}"

    # internal components
    @_resources = Array()

  register: (resource) ->
    @_resources.push(resource) if resource not in @_resources

  schema: ->
    require('./methods/schema')(@)

  make_routes: (app, check_authentication) ->
    app.all('*', check_authentication) if check_authentication?
    app.get("#{@path}", @schema())
    for resource in @_resources
      # resource.schema
      if !resource.options.authorization.GET? then app.get("#{@path}#{resource.collection_path}/schema", resource.schema())
      else app.get("#{@path}#{resource.collection_path}/schema", resource.options.authorization.GET, resource.schema())

      # resource.find
      if !resource.options.authorization.GET? then app.get("#{@path}#{resource.collection_path}", resource.find())
      else app.get("#{@path}#{resource.collection_path}", resource.options.authorization.GET, resource.find())

      # resource.find_one
      if !resource.options.authorization.GET? then app.get("#{@path}#{resource.document_path}", resource.find_one())
      else app.get("#{@path}#{resource.document_path}", resource.options.authorization.GET, resource.find_one())

      # resource.create
      if !resource.options.authorization.POST? then app.post("#{@path}#{resource.collection_path}", resource.create())
      else app.post("#{@path}#{resource.collection_path}", resource.options.authorization.POST, resource.create())

      # resource.update
      if !resource.options.authorization.PUT? then app.put("#{@path}#{resource.document_path}", resource.update())
      else app.put("#{@path}#{resource.document_path}", resource.options.authorization.PUT, resource.update())

      # resource.destroy
      if !resource.options.authorization.DELETE? then app.delete("#{@path}#{resource.document_path}", resource.destroy())
      else app.delete("#{@path}#{resource.document_path}", resource.options.authorization.DELETE, resource.destroy())


module.exports = Api
