_ = require 'lodash'

ref2resource = require('./dry').ref2resource
self_uri = require('./dry').self_uri

respond = require '../../responses'


module.exports = (Resource) ->
  (req, res) ->
    # set query params
    limit = req.query.limit or Resource.options.limit
    skip = req.query.skip or undefined
    sort = req.query.sort or undefined

    # parse limit and skip to integers
    try limit = parseInt limit
    catch err then return respond.bad_request res, err.message
    if skip
      try skip = parseInt skip
      catch err then return respond.bad_request res, err.message

    # make query hash and clean it up
    query = _.merge req.query, req.params
    where = _.transform query, (conditions, value, key) ->
      if key not in ['limit', 'skip', 'sort']
        conditions[key] = value

    # parse query values to correct type
    for key, value of where
      where[key] = JSON.parse value

    # execute query
    Resource.model.find(where).sort(sort).skip(skip).limit(limit).exec (err, models) ->
      # validation failure
      return respond.bad_request(res, err.errors) if err

      # iterate collection and add sugar
      for model in models
        model._doc = ref2resource(model._doc, Resource.options.refs) if Resource.options.refs?
        model._doc.resource_uri = self_uri(model._doc, Resource.collection_path)

      # prime response body
      body = {}

      # build body meta object
      body.meta = {}
      body.meta.page_size = limit

      # get collection count
      Resource.model.count where, (err, count) ->
        # really should err - if it does, throw a server error and inspect
        return respond.server_error(res, err) if err
        # assign count
        body.meta.count = count

        # build `next` uri
        if body.meta.count > limit and body.meta.count > skip + limit
          if skip? then new_skip = skip + limit
          else new_skip = limit
          next = "#{Resource.options.path}?"
          next += "sort=#{sort}&" if sort
          next += "skip=#{new_skip}&limit=#{limit}"
        else next = null
        body.meta.next = next

        # build `previous` uri
        if skip? and skip != 0
          if (skip - limit) < 0 then new_skip = 0
          else new_skip = skip - limit
          previous = "#{Resource.options.path}?"
          previous += "sort=#{sort}&" if sort
          previous += "skip=#{new_skip}&limit=#{limit}"
        else previous = null
        body.meta.previous = previous

        # add in collection
        body.objects = models

        # send response
        respond.ok(res, body)
