diff = require('deep-diff').diff

ref2resource = require('./dry').ref2resource

respond = require '../../responses'


module.exports = (Resource) ->
  (req, res) ->
    # find the existing document using mongoose model method
    Resource.model.findById req.params._id, (err, existing_model) ->
      # internal error
      return respond.server_error(res, err) if err
      # 404
      return respond.not_found(res, "`#{Resource.model.modelName}`@`#{req.params._id}` not found") if !existing_model?

      # update
      Resource.model.findByIdAndUpdate req.params._id, req.body, (err, updated_model) ->
        # validation errors
        return respond.not_acceptable(res, err.errors) if err

        # get model delta
        delta = diff(existing_model._doc, updated_model._doc)

        # add sugar to updated model
        updated_model._doc = ref2resource(updated_model._doc, Resource.options.refs) if Resource.options.refs?
        updated_model._doc.resource_uri = self_uri(updated_model._doc, Resource.collection_path)

        # emit event
        Resource.emitter.emit('updated', updated_model._id, JSON.stringify(delta))

        # respond
        respond.ok(res, updated_model)
