_ = require 'lodash'
EventEmitter = require('events').EventEmitter


# function to clean up and semi-slug resource path
clean_path = (path) ->
  path = path.pyslice(1) if path.starts_with('/')
  path = path.pyslice(0,-1) if path.ends_with('/')
  path = path.toLowerCase()
  path = path.replace(' ', '-')


class Resource

  constructor: (@model, path, options) ->
    # default options
    @options = {
      limit: 20
      refs: {}
      authorization: {
        GET: null
        POST: null
        PUT: null
        DELETE: null
      }
    }

    # path spec
    path = clean_path(path)
    @collection_path = "/#{path}"
    @document_path = "#{@collection_path}/:_id"

    # overwrite options
    if @options? then _.merge(@options, options)
    else @options = default_options

    # event emitter
    @emitter = new EventEmitter()

  find: ->
    require('./methods/find')(@)

  find_one: ->
    require('./methods/find_one')(@)

  create: ->
    require('./methods/create')(@)

  update: ->
    require('./methods/update')(@)

  destroy: ->
    require('./methods/destroy')(@)

  schema: ->
    require('./methods/schema')(@)


module.exports = Resource
