http = require 'http'
request = require 'request'

auth_app = require('./helpers/app').auth_app
Clique = require('./helpers/schemas').Clique
Person = require('./helpers/schemas').Person
simple_authentication_check = require './helpers/authentication'
deny_all_authorization = require('./helpers/authorization').deny_all
allow_all_authorization = require('./helpers/authorization').allow_all

Api = require('../../lib').Api
Resource = require('../../lib').Resource


URI = "http://localhost:3001/api/v1"


describe 'Auth Functional Tests', ->

  auth_server = {}

  before ->
    auth_clique_resource = new Resource(Clique, 'cliques',
      {
        authorization: { GET: allow_all_authorization, POST: allow_all_authorization, PUT: allow_all_authorization, DELETE: allow_all_authorization }
      })
    auth_person_resource = new Resource(Person, 'people',
      {
        refs: { cliques: auth_clique_resource }
        authorization: { GET: deny_all_authorization, POST: deny_all_authorization, PUT: deny_all_authorization, DELETE: deny_all_authorization }
      })

    auth_api = new Api()
    auth_api.register(auth_clique_resource)
    auth_api.register(auth_person_resource)
    auth_api.make_routes(auth_app, simple_authentication_check)
    auth_server = http.createServer(auth_app)
    auth_server.listen auth_app.get('port')

  after (done) ->
    auth_server.close(done)

  it 'fails authentication when no creds presented', (done) ->
    request { method: 'GET', uri: "#{URI}" }, (err, resp, body) ->
      resp.statusCode.should.equal 401
      done()

  it 'passes authentication when proper creds presented', (done) ->
    request { method: 'GET', uri: "#{URI}", headers: { username: 'fulton', password: 'func-tests' } }, (err, resp, body) ->
      resp.statusCode.should.equal 200
      done()

  it 'fails authentication when bad creds presented', (done) ->
    request { method: 'GET', uri: "#{URI}", headers: { username: 'bob', password: 'func-tests' } }, (err, resp, body) ->
      resp.statusCode.should.equal 401
      done()

  it 'fails deny all authorization', (done) ->
    request { method: 'GET', uri: "#{URI}/people", headers: { username: 'fulton', password: 'func-tests' } }, (err, resp, body) ->
      resp.statusCode.should.equal 401
      done()

  it 'passes permissive authorization with correct creds', (done) ->
    request { method: 'GET', uri: "#{URI}/cliques", headers: { username: 'fulton', password: 'func-tests' } }, (err, resp, body) ->
      resp.statusCode.should.equal 200
      done()
