http = require 'http'
request = require 'request'

app = require('./helpers/app').app
Clique = require('./helpers/schemas').Clique
Person = require('./helpers/schemas').Person

Api = require('../../lib').Api
Resource = require('../../lib').Resource


URI = "http://localhost:3000/api/v1"


describe 'Basic Functional Tests', ->

  server = {}
  CLIQUE_ID = null

  before ->
    clique_resource = new Resource(Clique, 'cliques')
    person_resource = new Resource(Person, 'people', { refs: { cliques: clique_resource } })

    api = new Api()
    api.register(clique_resource)
    api.register(person_resource)
    api.make_routes(app)
    server = http.createServer(app)
    server.listen app.get('port')

  after (done) ->
    server.close(done)

  it 'all api paths are canonical', (done) ->
    routes = []
    for route in app._router.stack
      routes.push(route.route.path) if route.route? and route.route.path?
    for route in routes
      route.pyslice(0, '/api/v1'.length).should.equal '/api/v1'
      parts = route.split('/')
      parts[1].should.equal 'api'
      parts[2].should.equal 'v1'
      if parts[3]
        parts[3].should.match /cliques|people/
    done()

  describe 'POST (clique)', ->

    err = null
    resp = null
    body = null

    before (done) ->
      request { method: 'POST', uri: "#{URI}/cliques", json: {name:'Developers'} }, (_err, _resp, _body) ->
        err = _err
        resp = _resp
        body = _body
        CLIQUE_ID = body._id
        done()

    it 'status code should be 201', (done) ->
      resp.statusCode.should.equal 201
      done()

    it 'body should have added resource_uri property', (done) ->
      body.should.have.property 'resource_uri'
      done()

  describe 'GET collection (clique)', ->

    err = null
    resp = null
    body = null

    before (done) ->
      request { method: 'GET', uri: "#{URI}/cliques" }, (_err, _resp, _body) ->
        err = _err
        resp = _resp
        body = JSON.parse(_body)
        done()

    it 'status code should be 200', (done) ->
      resp.statusCode.should.equal 200
      done()

    it 'body should have meta property', (done) ->
      body.should.have.property 'meta'
      done()

    it 'body should have objects property', (done) ->
      body.should.have.property 'objects'
      done()

    it 'all documents in objects should have a resource_uri property', (done) ->
      for obj in body.objects
        obj.should.have.property 'resource_uri'
      done()

  describe 'GET document (clique)', ->

    err = null
    resp = null
    body = null

    before (done) ->
      request { method: 'GET', uri: "#{URI}/cliques/#{CLIQUE_ID}" }, (_err, _resp, _body) ->
        err = _err
        resp = _resp
        body = JSON.parse(_body)
        done()

    it 'status code should be 200', (done) ->
      resp.statusCode.should.equal 200
      done()

  describe 'PUT (clique)', ->

    err = null
    resp = null
    body = null

    before (done) ->
      request { method: 'PUT', uri: "#{URI}/cliques/#{CLIQUE_ID}", json: { characteristics: ['awesome'] } }, (_err, _resp, _body) ->
        err = _err
        resp = _resp
        body = _body
        done()

    it 'status code should be 200', (done) ->
      resp.statusCode.should.equal 200
      done()

    it 'body should reflect changes', (done) ->
      body.characteristics[0].should.equal 'awesome'
      done()

  describe 'POST (person -- ref chaining)', ->

    err = null
    resp = null
    body = null

    before (done) ->
      request { method: 'POST', uri: "#{URI}/people", json: { name: { first: 'Fulton', last: 'Api'}, cliques: [ CLIQUE_ID ] } }, (_err, _resp, _body) ->
        err = _err
        resp = _resp
        body = _body
        done()

    it 'cliques should be resource uris, not straight object ids', (done) ->
      body.cliques[0].should.equal "/cliques/#{CLIQUE_ID}"
      done()

  describe 'DELETE', ->

    err = null
    resp = null
    body = null

    before (done) ->
      request { method: 'DELETE', uri: "#{URI}/cliques/#{CLIQUE_ID}" }, (_err, _resp, _body) ->
        err = _err
        resp = _resp
        body = _body
        done()

    it 'status code should be 200', (done) ->
      resp.statusCode.should.equal 204
      done()

    it 'body should be empty', (done) ->
      body.should.equal ''
      done()
